package pomodoro

import (
	"git.smsvc.net/pomodoro/GoTomato/internal/broadcast"
	"git.smsvc.net/pomodoro/GoTomato/pkg/models"
	"sync"
)

var pomodoroResetChannel = make(chan bool, 1)
var pomodoroPauseChannel = make(chan bool, 1)
var pomodoroResumeChannel = make(chan bool, 1)

var mu sync.Mutex // to synchronize access to shared state

// RunPomodoro iterates the Pomodoro work/break sessions.
func RunPomodoro(config models.GoTomatoPomodoroConfig) {
	mu.Lock()
	broadcast.Message.Ongoing = true
	broadcast.Message.Paused = false
	mu.Unlock()

	broadcast.Message.TotalSession = config.Sessions

	for session := 1; session <= config.Sessions; session++ {
		broadcast.Message.Session = session
		if !startTimer(config.Work, "Work") {
			break
		}
		if session == config.Sessions {
			if !startTimer(config.LongBreak, "LongBreak") {
				break
			}
		} else {
			if !startTimer(config.ShortBreak, "ShortBreak") {
				break
			}
		}
	}

	mu.Lock()
	broadcast.Message.Ongoing = false
	mu.Unlock()
}

// ResetPomodoro resets the running Pomodoro timer.
func ResetPomodoro() {
	// Send a reset signal to stop any running timers
	pomodoroResetChannel <- true

	mu.Lock()
	broadcast.Message.Ongoing = false
	broadcast.Message.Paused = false
	mu.Unlock()

	// Reset message
	broadcast.Message.Mode = "none"
	broadcast.Message.Session = 0
	broadcast.Message.TotalSession = 0
	broadcast.Message.TimeLeft = 0
}

func PausePomodoro() {
	mu.Lock()
	broadcast.Message.Paused = true
	mu.Unlock()

	pomodoroPauseChannel <- true
}

func ResumePomodoro() {
	mu.Lock()
	broadcast.Message.Paused = false
	mu.Unlock()
	pomodoroResumeChannel <- true
}

func IsPomodoroOngoing() bool {
	mu.Lock()
	defer mu.Unlock() // Ensures that the mutex is unlocked after the function is done
	return broadcast.Message.Ongoing
}

func IsPomodoroPaused() bool {
	mu.Lock()
	defer mu.Unlock() // Ensures that the mutex is unlocked after the function is done
	return broadcast.Message.Paused
}
